# sigLLs: significant probesetIds or locuslinkids (if allChipLLs specified)
# allChipLLs: all LLs on the chip
# lib: library with LLIds to GO mappings or ProbesetIDS to GO Mappings
LLGOHyper <- function (sigLLs, lib="hgu95av2", what = "MF", allChipLLs = NULL) 
{
    require(GOstats) || stop("need data package GOstats")
    require(lib, character.only = TRUE) || stop("need data package", lib)
    getDataEnv <- function(name, lib) {
        get(paste(lib, name, sep = ""), mode = "environment")
    }
    countLLinGO <- function(x) {
        if (length(x) == 0 || is.na(x)) 
            return(NA)
        if (is.null(allChipLLs)) {
         lls <- unique(unlist(mget(x, getDataEnv("LOCUSID", lib))))
        } else {
         lls=as.numeric(unlist(x)) %in% allChipLLs
         lls=unique(as.numeric(unlist(x))[lls])
        }
        lls <- lls[!is.na(lls)]
        lls
    }
    if (any(duplicated(sigLLs)) || any(duplicated(allChipLLs))) 
        stop("input IDs must be unique")
    match.arg(what, c("MF", "BP", "CC"))
# if we have to deal with probesetids first convert them to locusids
    if (is.null(allChipLLs)) {
     cLLs <- unlist(as.list(getDataEnv("LOCUSID", lib)))
     ourLLs <- cLLs[match(sigLLs, cLLs)]
     goV <- as.list(getDataEnv("GO2ALLPROBES", lib))
    } else {
      goV =as.list(getDataEnv("GO2LL",lib))  # for {human,rat,mouse}LLMappings
      #lib="GO"; goV =as.list(getDataEnv("LOCUSID",lib))  # for GO package, which contains all species LocusID (Entrez Gene) data
      #lib="hgu133plus2"; goA <- as.list(getDataEnv("GO2ALLPROBES", lib)); goV=lapply(goA,function(x) unique(unlist(mget(x,envir=hgu133plus2LOCUSID))))
    }
    whWeHave <- sapply(goV, function(y) {
        if (is.na(y) || length(y) == 0) 
            return(FALSE)
        if (is.null(allChipLLs)) {
          lls = unique(unlist(mget(y, getDataEnv("LOCUSID", lib),
            ifnotfound = NA)))
        } else lls=y;
        any(sigLLs %in% lls)
    })
    goV <- goV[whWeHave]
    goCat <- unlist(getGOOntology(names(goV)))
    goodGO <- goCat == what
    mm <- match(names(goCat), names(goV))
    mm <- mm[goodGO]
    goV <- goV[mm]
    if (length(goV) == 0) 
      return(list(pvalues = numeric(0), goCounts = numeric(0), 
             chip=lib, go2Affy=goV, intCounts = numeric(0), numLL = 0, 
             numInt = 0, intLLs=sigLLs))
    if (length(goV) > 1) goVcts = sapply(goV, countLLinGO)
    else {
      goVcts = list(countLLinGO(goV[[1]]))
      names(goVcts) <- names(goV)
    }
    bad <- sapply(goVcts, function(x) (length(x) == 1 && is.na(x)))
    goVcts = goVcts[!bad]
    goV = goV[!bad]
    allChipLLs <- unique(unlist(goVcts))
    nLL <- length(allChipLLs)
    goCounts <- sapply(goVcts, length)
    sigLLs <- unique(sigLLs[!is.na(sigLLs)])
    ours <- sigLLs[!duplicated(sigLLs)]
    whGood <- ours[ours %in% allChipLLs]
    nInt = length(whGood)
    if (nInt == 0) 
        warning("no interesting genes found")
    useCts <- sapply(goVcts, function(x) sum(whGood %in% x))
    if (length(useCts) == 0) 
    return(list(pvalues = numeric(0), goCounts = numeric(0), 
        chip=lib, go2Affy=goV, intCounts = numeric(0), numLL = nLL, 
        numInt = nInt, intLLs=sigLLs))
    pvs <- phyper(useCts - 1, nInt, nLL - nInt, goCounts, lower.tail = FALSE)
    ord <- order(pvs)
    return(list(pvalues = pvs[ord], goCounts = goCounts[ord], 
        chip=lib, go2Affy=goV, intCounts = useCts[ord], numLL = nLL, 
        numInt = nInt, intLLs=sigLLs))
}

