# getGOTable (probe, chip)
#
# takes:	probe - affymetrix probe id
#		chip  - affymetrix chip name
#
# returns:	a String of a HTML-Table that
#		consists of GO-Terms for a specific probe, ordered by ontology
#

getGOTable <- function (x, y) {
  # y contains the chip name if class(X)
  # is character (x then is the name of a probe-set).
  # If class(x) is aafTable, y contains a position 
  # (the row in the table).

  if (class(x) == "character") { goInfo <- aafGO(x, y)[[1]] }
  if (class(x) == "aafTable") { goInfo <- x[y, "Gene Ontology"]@table[[1]][[1]] }

  if (length(goInfo) == 0) return("n/a")

  goTableStr <- paste("<table border=0>\n<tr valign='top'><td>CC:</td><td>",sep="")
  for (i in 1:length(goInfo)) {
    if(goInfo[[i]]@type=="Cellular Component") {
      goTableStr <- paste(goTableStr,"<a href='", getURL(goInfo[[i]]),
                          "' target='annotation' title='", goInfo[[i]]@name, "'>", 
                          goInfo[[i]]@name, "</a><br>", sep="")
    }
  }
  goTableStr <- paste(goTableStr,"</td></tr>\n<tr valign='top'><td>BP:</td><td>",sep="")
  for (i in 1:length(goInfo)) {
    if(goInfo[[i]]@type=="Biological Process") {
      goTableStr <- paste(goTableStr,"<a href='", getURL(goInfo[[i]]),
                          "' target='annotation' title='", goInfo[[i]]@name, "'>", 
                          goInfo[[i]]@name, "</a><br>", sep="")
    }
  }
  goTableStr <- paste(goTableStr,"</td></tr>\n<tr valign='top'><td>MF:</td><td>",sep="")
  for (i in 1:length(goInfo)) {
    if(goInfo[[i]]@type=="Molecular Function") {
      goTableStr <- paste(goTableStr,"<a href='", getURL(goInfo[[i]]),
                          "' target='annotation' title='", goInfo[[i]]@name, "'>", 
                          goInfo[[i]]@name, "</a><br>", sep="")
    }
  }
  goTableStr <- paste(goTableStr,"</td></tr></table>", sep="")

  return(goTableStr)
}
