gokeggLister <- function(x, chip, groupname="first group", ngenes=NULL, max.q=NULL, 
                         gomf=TRUE, gobp=TRUE, gocc=TRUE, kegg=TRUE, outputPath=".",
                         max.p=0.01, links=FALSE, legend=FALSE, allChipLLs=NULL, 
                         is.upreg=NULL, bw.colors=FALSE, fontpath=NULL) {

  if (is.null(chip)) return("<p><ul><li>No annotation data available.</li></ul></p>")
  require(chip, character.only = TRUE) || stop("need data package", chip)
  require(GO)
  require(GOstats)

  # auxiliary functions
  goURL <- function(id, what=NULL) {
    if (is.null(what))
      return(paste("http://www.godatabase.org/cgi-bin/amigo/go.cgi?open_0=", id, sep=""))
    return(paste("javascript:openLink(\'links/", what, "_", sub(":","_",id), 
                 ".html\',\'link\')", sep=""))
  }

  goLink <- function(ids, what=NULL) {
    urls <- sapply(ids, goURL, what, USE.NAMES=FALSE)
    return(paste('<a href="', urls, '">', ids, '</a>', sep=""))
  }

  if (is.null(fontpath))
    fontpath=paste('/usr/lib/X11/fonts/misc:',
                   '/usr/lib/X11/fonts/misc/75dpi:',
                   '/usr/lib/X11/fonts/100dpi:', 
                   '/usr/lib/X11/fonts/TTF:',
                   '/usr/lib/X11/fonts/Type1:', 
                   '/usr/local/cups/share/cups/fonts:',
                   '/usr/share/fonts/default/Type1:',
                   '/usr/local/share/fonts/TrueType:',
                   '/usr/local/grace/fonts/type1', sep="")
  graphHeader <- paste("digraph {\ncenter=TRUE;\nrankdir=LR;\nsize=\"10,10\";\n",
                       "fontpath=\"",fontpath,"\";\n",
                       "node [style=filled,fontname=\"Helvetica\",fontsize=\"12\" ",
                       "height=0.1 width=0.1];\n", sep="")

  oldDigits <- getOption("digits")
  options(digits=3)
  on.exit(options(digits=oldDigits))

  if (! is.null(allChipLLs)) kegg=FALSE;

  # GO analysis for Genelists
  hyperGHTML <- function(LL, what, outputPath, max.p, allChipLLs=NULL) {
    tmp <- LLGOHyper(LL, lib=chip, what=what, allChipLLs)
    if (links) tmpWhat <- what else tmpWhat <- NULL
    if (length(tmp$pvalues) == 0) {
      cat("<UL><LI>no worthwhile", what, "annotations found</LI></UL>\n")
      return(invisible(NULL))
    }
    tmpFrame <- data.frame("GO.ID" = goLink(names(tmp$pvalues), tmpWhat),
                           "GO.Term" = as.character(getGOTerm(as.character(names(tmp$pvalues)))[[1]]),
                           "p.value" = format.pval(tmp$pvalues),
                           "int.Count" = tmp$intCounts,
                           "GO.Count" = tmp$goCounts)

    select <- tmp$pvalues < max.p & tmp$intCounts > 1

    if (sum(select) == 0) {
      cat("<UL><LI>no worthwhile", what, "annotations found</LI></UL>\n")
      return(invisible(NULL))
    }

    cat("<P><UL>\n",
        "<LI>",tmp$numLL," LocusLink IDs have annotations in category '",
        what,"'</LI>\n",
        "<LI>",tmp$numInt," of these are in the above list</LI>\n</UL>\n", sep="")
    cat(cpdTable(tmpFrame[select,], spc="5", 
                 tbh=c("GO ID","GO Term","p-value", "int. Count","GO Count")), "\n</P>\n")

    # making GO graph

    if(sum(select) == 0) { no_p_values = TRUE
    } else {
      no_p_values = FALSE
      goids <- row.names( tmpFrame[select, ])
    }

    # decide whether the GO graph should be drawn or not
    if ( !is.null(outputPath) & !no_p_values ) {

      # select go kind
      if (what=="MF")
        graph <- GOGraph(goids, GOMFPARENTS)
      if (what=="CC")
        graph <- GOGraph(goids, GOCCPARENTS)
      if (what=="BP")
        graph <- GOGraph(goids, GOBPPARENTS)

      # path and filename for .dot file
      filename = paste(outputPath, "/go", what, "Graph.dot", sep="")

      # write a .dot-file since I don't manage to use Rgraphviz
      cat(graphHeader, file=filename)
      for (n in nodes(graph)) {
        if (n == "all") next
        curColor <- ifelse(n %in% goids, "yellow", "white")
        if (links) tmpWhat <- what else tmpWhat <- NULL
        link <- goURL(n, tmpWhat)
        tooltip <- as.character(getGOTerm(n))
        if (n %in% rownames(tmpFrame)) {
          tooltip <- paste(tooltip,", p-value: ", format.pval(tmp$pvalues[n]), sep="")
        }

        cat('"', n, '" [fillcolor="', curColor, '", URL="', link, '" tooltip="',
            tooltip, '"];\n', sep="", file=filename, append=TRUE)
      }
      edges <- edges(graph)
      for (i in 1:length(edges)) {
        for (n in edges[[i]]) {
          if (n == "all") next
            cat('"', n, '" -> "', names(edges)[i], "\"\n",
         
                sep="", file=filename, append=TRUE)
        }
      }
      cat("}\n", file=filename, append=TRUE)


      # path and filename for .png file and imagemap
      savepath <- paste(outputPath,"/go", what, "Graph.png", sep="")
      filename_html <- paste("go", what, "Graph.png", sep="")
      map_name <- paste("go", what, "Map", sep="")

      system(paste("dot -Tpng < ",filename," > ",savepath, sep=""))
      cmap <- system(paste("dot -Tcmap < ", filename, sep=""), intern=TRUE)
      cmap <- gsub(' alt="GO:[0-9]*a?" ', '', cmap)

      # insert graphs with imagemap in HTML
      cat("<p align=\"center\">\n")
      cat('<img src="', filename_html, '" usemap="#', map_name, 
          '" alt="Induced GO graph">\n', sep="")
      cat("</p>\n")
      cat('<map name="', map_name, '">\n', sep="")
      for (i in 1:length(cmap)) {
        cat(cmap[i], "\n")
      }
      cat('</map> \n')

      if(links)
        makeLinkSites(nodes(graph), what=what)
   }
  }


 
  if (is.null(allChipLLs)) {
    path.env <- get(paste(chip, "PATH", sep=""), paste("package:", chip, sep=""))
    allpaths <- length(path.env) - sum(is.na(unlist(as.list(path.env))))
    pathprobe.env <- get(paste(chip,"PATH2PROBE",sep=""), paste("package:",chip,sep=""))
  }

  # KEGG analysis for Genelists
  hyperKHTML <- function(probes, max.p) {
    kid <- mget(as.character(probes), path.env, ifnotfound=NA)
    unikid <- table(unlist(kid))
    kprobes <- sum(!is.na(kid))
    if (length(unikid) == 0) {
      cat("<UL><LI>no KEGG annotations found</LI></UL>\n")
      return(invisible(NULL))
    }

    pathNames <- as.character(mget(names(unikid), KEGGPATHID2NAME, ifnotfound=NA))
    unikid <- unikid[!is.na(pathNames)]
    pathNames <- pathNames[!is.na(pathNames)]
    pathLens <- sapply(mget(names(unikid), pathprobe.env,, ifnotfound=NA), length)
    allpaths <- length(path.env) - sum(is.na(unlist(as.list(path.env))))
    pvalues <- phyper(unikid - 1, pathLens, allpaths - pathLens,
                      sum(!is.na(unikid)), lower.tail=FALSE)
    KEGGFrame <- data.frame("KEGG ID" = names(unikid),
                            "Path Name" = pathNames,
                            "p.value" = as.numeric(as.numeric(pvalues)),
                            "int.Count" = as.numeric(unikid),
                            "KEGG.Count" = pathLens)
    #rownames(KEGGFrame) <- names(unikid)
    KEGGFrame <- KEGGFrame[order(pvalues), ]
    select <- KEGGFrame$p.value < max.p & KEGGFrame$int.Count > 1
    KEGGFrame$p.value <- format.pval(KEGGFrame$p.value)
    if (sum(select) == 0) {
      cat("<UL><LI>no worthwhile KEGG annotations found</LI></UL>\n")
      return(invisible(NULL))
    }

    cat("<P><UL><LI>Probes with KEGG annotations in above list: ", kprobes, "</LI>\n",
      "<LI>The chip holds ", allpaths, " probes annotated to ", length(pathprobe.env),
      " pathways</LI></UL>\n", sep="")
    cat(cpdTable(KEGGFrame[select,], spc="5", 
                 tbh=c("KEGG ID","Path Name","p.value", "Int.Count","KEGG.Count")),"\n")
    cat("</P>\n")
  }


  # GO analysis for twilight objects (2 genelists up and down regulated)
  hyperGTwilightHTML <- function(LL_upreg, LL_downreg, what, outputPath, max.p, allChipLLs=NULL) {
    # check if geneLists are empty
    if (is.null(LL_upreg)) {
      tmp_upreg <- NULL
      terms_upreg <- NULL
      go_ids_upreg = NULL
    } else {
      tmp_upreg <- LLGOHyper(LL_upreg, lib=chip, what=what, allChipLLs)
      if (length(tmp_upreg$pvalues) == 0) {
        tmp_upreg <- NULL
        terms_upreg <- NULL
        go_ids_upreg = NULL
      } else {
        terms_upreg <- as.character(getGOTerm(names(tmp_upreg$pvalues))[[1]])
        go_ids_upreg = names(tmp_upreg$pvalues)
      }
    }
    # check if geneLists are empty
    if (is.null(LL_downreg)) {
      tmp_downreg <- NULL
      terms_downreg <- NULL
      go_ids_downreg = NULL
    } else {
      tmp_downreg <- LLGOHyper(LL_downreg, lib=chip, what=what, allChipLLs)
      if (length(tmp_downreg$pvalues) == 0) {
        tmp_downreg <- NULL
        terms_downreg <- NULL
        go_ids_downreg = NULL
      } else {
        terms_downreg <- as.character(getGOTerm(names(tmp_downreg$pvalues))[[1]])
        go_ids_downreg = names(tmp_downreg$pvalues)
      }
    }

    # initialize storeing variables
    go.terms <- c("")
    rowNames <- c("")
    upreg.p.values <- 0
    upreg.int.Counts <- 0
    downreg.p.values <- 0
    downreg.int.Counts <- 0
    GO.Counts <- 0

    # gather infos concerning upregulated genes
    if (!is.null(go_ids_upreg)) {
      for(i in 1:length(go_ids_upreg)) {
        rowNames[i] <- go_ids_upreg[i]
        go.terms[i] <- terms_upreg[i]
        upreg.p.values[i] <- as.numeric(tmp_upreg$pvalues[i])
        upreg.int.Counts[i] <- tmp_upreg$intCounts[i]
        GO.Counts[i] <- tmp_upreg$goCounts[i]

        # genes can only occure once in upreg and/or downreg
        bool_select = go_ids_downreg == go_ids_upreg[i];
        if(sum(bool_select)==1) {
          downreg.p.values[i] <- as.numeric(tmp_downreg$pvalues[bool_select])
          downreg.int.Counts[i] <- tmp_downreg$intCounts[bool_select]
        }
        else {
          downreg.p.values[i] <- 1
          downreg.int.Counts[i] <- 0
        }
      }
    }

    # gather infos concerning downregulated genes
    counter = length(go.terms)
    if (!is.null(go_ids_downreg)) {
      for(i in 1:length(go_ids_downreg)) {
        if(sum(go_ids_upreg == go_ids_downreg[i])==0) {
          counter = counter+1
          rowNames[counter] <- go_ids_downreg[i]
          go.terms[counter] <- terms_downreg[i]
          downreg.p.values[counter] <- as.numeric(tmp_downreg$pvalues[i])
          downreg.int.Counts[counter] <- tmp_downreg$intCounts[i]
          GO.Counts[counter] <- tmp_downreg$goCounts[i]

          upreg.p.values[counter] <- 1
          upreg.int.Counts[counter] <- 0
        }
      }
    }

    # building dataframe
    if (links) tmp <- what else tmp <- NULL
    tmpFrame <- data.frame("GO.ID" = goLink(rowNames, tmp),
                           "GO.Term" = go.terms,
                           "upreg.p.value" = format.pval(upreg.p.values),
                           "upreg.int.Count" = upreg.int.Counts,
                           "downreg.p.value" = format.pval(downreg.p.values),
                           "downreg.int.Count" = downreg.int.Counts,
                           "GO.Count" = GO.Counts,
                           row.names = rowNames)
    names(upreg.p.values) <- rowNames
    names(downreg.p.values) <- rowNames

    select <- ((upreg.p.values < max.p & upreg.int.Counts > 1) |
              (downreg.p.values < max.p & downreg.int.Counts > 1))

    if(sum(select) == 0) {
      cat("<UL><LI>no worthwhile", what, "annotations found</LI></UL>\n")
      return(invisible(NULL))
    }

    cat("<P><UL>\n",
        "<LI>",tmp_upreg$numLL," LocusLink IDs have annotations in category '",
        what,"'</LI>\n",
        "<LI>", tmp_upreg$numInt + tmp_downreg$numInt, 
        " of these are in the above list</LI>\n", sep="")
    if (!is.na(groupname)) {
      cat("<LI>upreg means upregulated in group ", groupname, 
          " and downreg means downregulated in group ", groupname, 
          "</LI>\n", sep="")
    }
    tmp <- upreg.p.values[select]
    oo <- order(ifelse(tmp>max.p, max.p,tmp), -downreg.p.values[select])
    cat("</UL>\n", 
        cpdTable(tmpFrame[select,][oo,], spc="5", 
                 tbh=c("GO ID","GO Term","upreg. p-value", "upreg. int. Count", 
                       "downreg. p-value", "downreg. int. Count","GO Count")), 
                       "\n</P>\n")

    # making GO graph

    # set NaN values to FALSE
    for(i in 1:length(select)) {
      if(is.na(select[i])) select[i] <- FALSE
    }

    if(sum(select) == 0) {
      no_p_values = TRUE
    } else {
      no_p_values = FALSE
      goids <- row.names( tmpFrame[select, ]  )
    }

    # decide whether the GO graph should be drawn or not
    if ( !is.null(outputPath) & !no_p_values ) {

      # select go kind
      if (what=="MF")
        graph <- GOGraph(goids, GOMFPARENTS)
      if (what=="CC")
        graph <- GOGraph(goids, GOCCPARENTS)
      if (what=="BP")
        graph <- GOGraph(goids, GOBPPARENTS)

      # path and filename for .dot file
      filename = paste(outputPath, "/go", what, "Graph.dot", sep="")

      # write a .dot-file since I don't manage to use Rgraphviz
      cat(graphHeader, file=filename)

      # setting colors for nodes
      white           = "#FFFFFF"
      green           = "#00FF00"  # "#00FF00"
      d_green         = "#64FF64"  # "#00CD00"
      vd_green        = "#C8FFC8"  # "#009B00"
      red             = "#FF0000"  # "#FF0000"
      d_red           = "#FF6464"  # "#CD0000"
      vd_red          = "#FFC8C8"  # "#9B0000"
      red_green       = "#FFFF00"  # "#FFFF00"
      red_d_green     = "#E1BE23"  # "#FFCD00"
      red_vd_green    = "#FFBE41"  # "#FF9B00"
      d_red_green     = "#BEE123"  # "#CDFF00"
      d_red_d_green   = "#E1E146"  # "#CDCD00"
      d_red_vd_green  = "#FFFF64"  # "#CD9B00"
      vd_red_green    = "#BEFF41"  # "#9BFF00"
      vd_red_d_green  = "#E1FF64"  # "#9BCD00"
      vd_red_vd_green = "#FFFF82"  # "#9B9B00"

      # shades of gray for nodes
      gray <- "#787878"
      d_gray <- "#B0B0B0"
      vd_gray <- "#DEDEDE"

      # shapes for nodes
      defShape <- "ellipse"
      upShape <- "house"
      downShape <- "invhouse"
      updownShape <- "rectangle"

      # write color legend in dot/png
      if (bw.colors) {
        writeColorLegend(graphHeader, max.p, upShape, updownShape, downShape, defShape,
                         gray,gray,gray,gray,
                         gray,d_gray,d_gray,d_gray,
                         gray,d_gray,vd_gray,vd_gray,
                         gray,d_gray,vd_gray,white)
      } else {
        writeColorLegend(graphHeader, max.p, "ellipse", "ellipse", "ellipse", "ellipse",
                         red_green,d_red_green,vd_red_green,green,
                         red_d_green,d_red_d_green,vd_red_d_green,d_green,
                         red_vd_green,d_red_vd_green,vd_red_vd_green,vd_green,
                         red,d_red,vd_red,white)
      }
      for (n in nodes(graph)) {
        if (n == "all") next

        bool_select <- rowNames==n
        if(sum(bool_select)==1) {

          # choose color and shape
          curColor <- white
          curShape <- defShape

          if( upreg.p.values[bool_select]<max.p & !is.na(upreg.p.values[bool_select]) ) {
            if (bw.colors) {
              curColor <- vd_gray
              curShape <- upShape
            } else curColor <- vd_green
          }
          if(upreg.p.values[bool_select]<max.p/10 & !is.na(upreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- d_gray
              curShape <- upShape
            } else curColor <- d_green
          }
          if(upreg.p.values[bool_select]<max.p/100 & !is.na(upreg.p.values[bool_select])){
            if (bw.colors) {
              curColor <- gray
              curShape <- upShape
            } else curColor <- green
          }

          if(downreg.p.values[bool_select]<max.p & !is.na(downreg.p.values[bool_select])){
            if (bw.colors) {
              curColor <- vd_gray
              curShape <- downShape
            } else curColor <- vd_red
          }
          if(downreg.p.values[bool_select]<max.p/10 & 
             !is.na(downreg.p.values[bool_select])){
            if (bw.colors) {
              curColor <- d_gray
              curShape <- downShape
            } else curColor <- d_red
          }
          if(downreg.p.values[bool_select]<max.p/100 & 
             !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- gray
              curShape <- downShape
            } else curColor <- red
          }

          if(upreg.p.values[bool_select]<max.p & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p &  
             !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- vd_gray
              curShape <- updownShape
            } else curColor <- vd_red_vd_green
          }
          if(upreg.p.values[bool_select]<max.p & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p/10 &  
             !is.na(downreg.p.values[bool_select])) { 
            if (bw.colors) {
              curColor <- vd_gray
              curShape <- updownShape
            } else curColor <- d_red_vd_green
          }

          if(upreg.p.values[bool_select]<max.p & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p/100 &  
             !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- d_gray
              curShape <- updownShape
            } else curColor <- red_vd_green
          }
          if(upreg.p.values[bool_select]<max.p/10 & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p &  
             !is.na(downreg.p.values[bool_select]) ) {
            if (bw.colors) {
              curColor <- vd_gray
              curShape <- updownShape
            } else curColor <- vd_red_d_green
          }
          if( upreg.p.values[bool_select]<max.p/10 & 
             !is.na(upreg.p.values[bool_select]) & 
              downreg.p.values[bool_select]<max.p/10 &  
              !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- d_gray
              curShape <- updownShape
            } else curColor <- d_red_d_green
          }
          if(upreg.p.values[bool_select]<max.p/10 & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p/100 & 
             !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- d_gray
              curShape <- updownShape
            } else curColor <- red_d_green
          }
          if(upreg.p.values[bool_select]<max.p/100 & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p &  
             !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- d_gray
              curShape <- updownShape
            } else curColor <- vd_red_green
          } 
          if(upreg.p.values[bool_select]<max.p/100 & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p/10 &  
             !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- d_gray
              curShape <- updownShape
            } else curColor <- d_red_green
          }
          if(upreg.p.values[bool_select]<max.p/100 & 
             !is.na(upreg.p.values[bool_select]) & 
             downreg.p.values[bool_select]<max.p/100 &  
             !is.na(downreg.p.values[bool_select])) {
            if (bw.colors) {
              curColor <- gray
              curShape <- updownShape
            } else curColor <- red_green
          }
        }


        if (links) tmp <- what else tmp <- NULL
        onclick <- goURL(n, tmp)
        tooltip <- as.character(getGOTerm(n))
        if (n %in% rownames(tmpFrame)) {
          tooltip <- paste(tooltip,", p-values: ", format.pval(upreg.p.values[n]),
                           " (up in ",groupname,"), ", format.pval(downreg.p.values[n]),
                           " (down in ",groupname,")", sep="")
        }

        cat('"', n, '" [fillcolor="', curColor, '" shape="', curShape, 
            '" URL="',onclick,'", tooltip="', tooltip, '"];\n', sep="", 
            file=filename, append=TRUE)
      }

      edges <- edges(graph)
      for (i in 1:length(edges)) {
        for (n in edges[[i]]) {
          if (n == "all") next
            cat('"', n, '" -> "', names(edges)[i], "\"\n",
                sep="", file=filename, append=TRUE)
        }
      }
      cat("}\n", file=filename, append=TRUE)



      savepath <- paste(outputPath,"/go", what, "Graph.png", sep="")
      filename_html <- paste("go", what, "Graph.png", sep="")
      map_name <- paste("go", what, "Map", sep="")


      system(paste("dot -Tpng < ",filename," > ",savepath, sep=""))
      cmap <- system(paste("dot -Tcmap < ", filename, sep=""), intern=TRUE)
      cmap <- gsub(' alt="GO:[0-9]*a?" ', '', cmap)

      # insert graphs with imagemap in HTML
      cat("<p align=\"center\">\n")
      cat('<img src="', filename_html, '" usemap="#',map_name,
          '" alt="Induced GO Graph for ', what, '">\n', sep="")
      cat("</p>\n")
      cat('<map name="', map_name, '">\n', sep="")
      for (i in 1:length(cmap)) {
        cat(cmap[i], "\n")
      }
      cat('</map> \n')


      # graph legend
      if(legend) {

        if(what == "BP")
          fn <- paste("BP_legend.html", sep="")
        if(what == "MF")
          fn <- paste("MF_legend.html", sep="")
        if(what == "CC")
          fn <- paste("CC_legend.html", sep="")

        cat('<a OnClick="openLink(\'', fn, 
            '\',\'legend\')" style="cursor:pointer" >Legend</a>\n', sep="")
        cat(' | <a OnClick="openLink(\'color.legend.png\',\'legend\')" style="cursor:pointer" >Color Code</a>\n', sep="")

        LegendHTML <- paste("<html>\n<header>\n<title>Legend</title>\n",
                            "<link REL=\"stylesheet\" HREF=\"/includes/style.css\"",
                            " TYPE=\"text/css\">\n</head>\n<body>\n<h2>",
                            what, " GO Graph Legend</h2>\n", sep="")

        legend_frame <- tmpFrame[nodes(graph),]
        legend_frame <- legend_frame[!is.na(legend_frame[,1]),]
        oo <- order(legend_frame[,1])

        LegendHTML <- paste(LegendHTML, 
                            cpdTable(legend_frame[oo,], spc="5", 
                                     tbh=c("GO.ID","GO.Term","upreg.p.value", 
                                           "upreg.int.Count", "downreg.p.value", 
                                           "downreg.int.Count","GO.Count")), 
                            "\n</body>\n</html>\n", sep="")

        write(LegendHTML, file=fn)
      }

      if(links)
        makeLinkSites(nodes(graph), what=what)

    }
  }

  getDataEnv <- function(name, lib) {
    get(paste(lib, name, sep = ""), mode = "environment")
  }


  makeLinkSites <- function(goIds, what="NA") {

    # alle GO-Terms aufm chip
    if (is.null(allChipLLs)) {
      allGOTerms <- as.list(getDataEnv("GO2ALLPROBES", chip))
    } else allGOTerms <- as.list(getDataEnv("GO2LL", chip))

    for (i in 1:length(goIds)) {
      if (goIds[i] == "all") next

      html <- paste("<html>\n<header>\n<title>List of Genes for ", goIds[i], "</title>\n",
                    "<link REL=\"stylesheet\" HREF=\"/includes/style.css\"", 
                    " TYPE=\"text/css\">\n</head>\n<body>\n",
                    "<h2>List of Genes for ", goLink(goIds[i]), "<br>'", 
                    getGOTerm(goIds[i]), "'</h2>\n<ul>\n", sep="")

      act_good_genes = c("");
      count = 0

      act_genes <- unique(as.vector(unlist(allGOTerms[goIds[i]])));

      # checking if selected genes (act_genes) are in the experimental data
      if (class(x) == "twilight") {
        # this is done by checking if the specific gene has a observed value in result
        obs_values <- x$result[act_genes,1]
        no_nas <- !is.na(obs_values)
        act_genes = act_genes[no_nas]
      } else {
        # if don't have twilight results, we are interested
        # in whether the genes are in 'x'
        act_genes <- intersect(as.character(x), act_genes)
      }

  if (is.null(allChipLLs)) {
      html <-  paste(html, geneLister(act_genes, chip=chip, annotLOCUS=FALSE, 
                                      annotGO=FALSE, annotUNIGENE=FALSE, 
                                      annotGENBANK=FALSE, annotLOCUSLINK=FALSE, 
                                      annotPUBMED=FALSE, annotKEGG=FALSE, annotOMIM=FALSE,
                                      annotGENECARDS=FALSE, links=TRUE),
                     "</body>\n</html>\n", sep="")
}

      fn = ""
      if(what == "BP")
        fn <- paste(outputPath,"/links/BP_", sub(":","_",goIds[i]), ".html", sep="")
      if(what == "MF")
        fn <- paste(outputPath,"/links/MF_", sub(":","_",goIds[i]), ".html", sep="")
      if(what == "CC")
        fn <- paste(outputPath,"/links/CC_", sub(":","_",goIds[i]), ".html", sep="")

      suppressWarnings(dir.create(paste(outputPath,"/links", sep="")))
      write(html, file=fn)
    }
  }



  # KEGG analysis for twilight objets (2 genelists up and down regulated)
  hyperKTwilightHTML <- function(probes_upreg, probes_downreg, max.p) {
    # find out about upregulated pathways
    kid_upreg <- mget(as.character(probes_upreg), path.env, ifnotfound=NA)
    kprobes_upreg <- sum(!is.na(kid_upreg))
    if (kprobes_upreg == 0) {
      unikid_upreg <- character(0)
      pathNames_upreg <- character(0)
      pathLens_upreg <- numeric(0)
      pvalues_upreg <- numeric(0)
    } else {
      unikid_upreg <- table(unlist(kid_upreg))
      pathNames_upreg <- as.character(mget(names(unikid_upreg), KEGGPATHID2NAME,
                                           ifnotfound=NA))
      unikid_upreg <- unikid_upreg[!is.na(pathNames_upreg)]
      pathNames_upreg <- pathNames_upreg[!is.na(pathNames_upreg)]
      pathLens_upreg <- sapply(mget(names(unikid_upreg), pathprobe.env, , ifnotfound=NA),
                               length)
      pvalues_upreg <- phyper(unikid_upreg - 1, pathLens_upreg, allpaths - pathLens_upreg,
                              kprobes_upreg, lower.tail=FALSE)
    }

    # find out about downregulated pathways
    kid_downreg <- mget(as.character(probes_downreg), path.env, ifnotfound=NA)
    kprobes_downreg <- sum(!is.na(kid_downreg))
    if (kprobes_downreg == 0) {
      unikid_downreg <- character(0)
      pathNames_downreg <- character(0)
      pathLens_downreg <- numeric(0)
      pvalues_downreg <- numeric(0)
    } else {
      unikid_downreg <- table(unlist(kid_downreg))
      pathNames_downreg <- as.character(mget(names(unikid_downreg), KEGGPATHID2NAME,
                                             ifnotfound=NA))
      unikid_downreg <- unikid_downreg[!is.na(pathNames_downreg)]
      pathNames_downreg <- pathNames_downreg[!is.na(pathNames_downreg)]
      pathLens_downreg <- sapply(mget(names(unikid_downreg), pathprobe.env,ifnotfound=NA),
                                 length)
      pvalues_downreg <- phyper(unikid_downreg - 1, pathLens_downreg,
                                allpaths - pathLens_downreg, kprobes_downreg,
                                lower.tail=FALSE)
    }

    if ((kprobes_upreg == 0) & (kprobes_downreg == 0)) {
      cat("<UL><LI>no KEGG annotations found</LI></UL>\n")
     return(invisible(NULL))
    }

    pathNames <- character(0)
    pvalues_up <- character(0)
    unikid_up <- character(0)
    pvalues_down <- character(0)
    unikid_down <- character(0)
    pathLens <- numeric(0)
    rowNames <- character(0)
    unik_ids_upreg <- names(unikid_upreg)
    unik_ids_downreg <- names(unikid_downreg)

    if (length(unik_ids_upreg) > 0) {
      for (i in 1:length(unik_ids_upreg)) {
        pathNames[i]   <- pathNames_upreg[i]
        pvalues_up[i]  <- as.numeric(pvalues_upreg[i])
        unikid_up[i]   <- unikid_upreg[i]
        pathLens[i] <- pathLens_upreg[i]
        rowNames[i]    <- unik_ids_upreg[i]

        bool_select <- unik_ids_downreg == unik_ids_upreg[i]
        if(sum(bool_select) == 1) {
          pvalues_down[i]  <- as.numeric(pvalues_downreg[bool_select])
          unikid_down[i]   <- unikid_downreg[bool_select]
        }
        else {
          pvalues_down[i]  <- 1
          unikid_down[i]   <- 0
        }
      }
    }
    counter = length(pathNames)
    if (length(unik_ids_downreg) > 0) {
      for (i in 1:length(unik_ids_downreg)) {
        if(sum(unik_ids_upreg == unik_ids_downreg[i]) == 0) {
          counter = counter+1
          pathNames[counter] <- pathNames_downreg[i]

          pvalues_down[counter]  <- as.numeric(pvalues_downreg[i])
          unikid_down[counter]   <- unikid_downreg[i]
          pathLens[counter] <- pathLens_downreg[i]

          pvalues_up[counter]  <- 1
          unikid_up[counter]   <- 0

          rowNames[counter] <- unik_ids_downreg[i]
        } else { # fill in the info in existing table entry
          j = which(unik_ids_upreg == unik_ids_downreg[i])
          if (length(j) > 1) stop("internal error!")
          pvalues_down[j]  <- as.numeric(pvalues_downreg[i])
          unikid_down[j]   <- unikid_downreg[i]
        }
      }
    }

    KEGGFrame <- data.frame("KEGG ID" = rowNames,
                            "Path Name" = pathNames,
                            "upreg.p.value" = as.numeric(pvalues_up),
                            "upreg.int.Count" = as.numeric(unikid_up),
                            "downreg.p.value" = as.numeric(pvalues_down),
                            "downreg.int.Count" = as.numeric(unikid_down),
                            "KEGG.Count" = pathLens)


    select <- (KEGGFrame$upreg.p.value <= max.p & KEGGFrame$upreg.int.Count > 1) | 
              (KEGGFrame$downreg.p.value <= max.p & KEGGFrame$downreg.int.Count > 1)

    if (sum(select) == 0) {
      cat("<UL><LI>no worthwhile KEGG annotations found</LI></UL>\n")
      return(invisible(NULL))
    }
    KEGGFrame <- KEGGFrame[select,]

    oo <- order(ifelse(KEGGFrame$upreg.p.value>max.p,max.p,KEGGFrame$upreg.p.value), 
                -KEGGFrame$upreg.p.value)
    KEGGFrame$upreg.p.value <- format.pval(KEGGFrame$upreg.p.value)
    KEGGFrame$downreg.p.value <- format.pval(KEGGFrame$downreg.p.value)
    KEGGFrame <- KEGGFrame[oo,]

    no_kids = sum(!is.na(kid_upreg)) + sum(!is.na(kid_downreg))

    cat("<P><UL>", 
        "<LI>Up regulated probes with KEGG annotations in above list: ", kprobes_upreg, "</LI>\n",
        "<LI>Down regulated probes with KEGG annotations in above list: ", kprobes_downreg, "</LI>\n",
      "<LI>The chip holds ", allpaths, " probes annotated to ", length(pathprobe.env), 
      " pathways</LI></UL>\n", sep="")
    cat(cpdTable(KEGGFrame, spc="5", tbh=c("KEGG ID", "Path Name", "upreg.p.value", "upreg.Int.Count", "downreg.p.value", "downreg.Int.Count", "KEGG.Count")), "\n")
    cat("</P>\n")
  }




  # decide whether to separate up and down regulated

  updown <- FALSE

  if (class(x) == "twilight") {
    updown <- TRUE
    if (!is.null(max.q)) { # use max.q to trim the list
      select <- x$result$qvalue < max.q
      x$result <- x$result[select,]
    }
    if (!is.null(ngenes)) {
      x$result <- x$result[1:min(nrow(x$result),ngenes),]
    }

    up <- x$result$observed > 0
    down <- x$result$observed < 0
    upreg <- rownames(x$result)[up]
    downreg <- rownames(x$result)[down]
  }
  else {
    if (class(x) != "character" && class(x) != "numeric" && class(x) != "integer") {
      warning("Don't know how to treat x")
      return(invisible(NULL))
    }
    else {
      if (length(x) == 0) return(invisible(NULL))
      if (is.null(is.upreg)) {
        if (!is.null(ngenes)) {
          x <- x[1:min(length(x),ngenes)]
        }
      } else {
        if (!is.null(ngenes)) {
          x <- x[1:min(length(x),ngenes)]
          is.upreg <- is.upreg[1:min(length(x),ngenes)]
        }
        updown <- TRUE
        upreg <- x[is.upreg]
        downreg <- x[!is.upreg]
      }
    }
  }

  # collect locus IDs
  if (is.null(allChipLLs)) {
    locus.env <- get(paste(chip, "LOCUSID", sep=""), paste("package:", chip, sep=""))
    if (updown) {
      LL_upreg <- unique(unlist(mget(upreg, locus.env, ifnotfound=NA)))
      LL_downreg <- unique(unlist(mget(downreg, locus.env, ifnotfound=NA)))
    } else {
      LL <- unique(unlist(mget(x, locus.env, ifnotfound=NA)))
    }
  } else {
    if (updown) {
      LL_upreg <- upreg
      LL_downreg <- downreg
    } else LL=x;
  }

  # write HTML file
  stamp <- as.numeric(Sys.time())
  .tmpFileName <<- tempfile()
  .oldErrOption <<- getOption("error")
  .tmpFileCleanup <<- function() {
    sink()
    file.remove(.tmpFileName)
    if (!is.null(.oldErrOption)) eval(.oldErrOption)
    options(error=.oldErrOption)
  }
  options(error=.tmpFileCleanup)

  sink(.tmpFileName)
  cat("<LINK REL='stylesheet' HREF='style.css' TYPE='text/css'>\n", sep="")
  if (updown) {
    LL_length = length(LL_upreg) + length(LL_downreg)
  } else {
    LL_length = length(LL)
  }
  cat("<UL><LI>", LL_length, " unique LocusLink IDs considered</LI>\n",sep="")
  if (is.null(allChipLLs)) {
    cat("<LI>on chip with ", length(path.env), " probesets</LI></UL>\n<UL>\n", sep="")
  } else {
    cat("<LI>on chip with ", length(allChipLLs), " unique LocusLink Ids </LI></UL>\n<UL>\n", sep="")
  }

  if (gomf) {
    cat("<LI><A HREF='#MF", stamp, "'>Molecular function</A></LI>\n", sep="")
  }
  if (gobp) {
    cat("<LI><A HREF='#BP", stamp, "'>Biological process</A></LI>\n", sep="")
  }
  if (gocc) {
    cat("<LI><A HREF='#CC", stamp, "'>Cellular component</A></LI>\n", sep="")
  }
  if (kegg) {
    cat("<LI><A HREF='#KEGG", stamp, "'>Pathways (KEGG)</A></LI>\n", sep="")
  }

  cat("</UL>\n")

  if (gomf) {
    if(updown) {
      cat("<A NAME='MF", stamp, "'></A>", sep="")
      cat("<H3>Molecular Function</H3>\n");
      hyperGTwilightHTML(LL_upreg, LL_downreg, "MF", outputPath, max.p, allChipLLs)
    } else {
      cat("<A NAME='MF", stamp, "'></A>", sep="")
      cat("<H3>Molecular Function</H3>\n");
      hyperGHTML(LL, "MF", outputPath, max.p, allChipLLs)
    }
  }
  if (gobp) {
    if(updown) {
      cat("<A NAME='BP", stamp, "'></A>", sep="")
      cat("<H3>Biological Process</H3>\n");
      hyperGTwilightHTML(LL_upreg, LL_downreg, "BP", outputPath, max.p, allChipLLs)
    } else {
      cat("<A NAME='BP", stamp, "'></A>", sep="")
      cat("<H3>Biological Process</H3>\n");
      hyperGHTML(LL, "BP", outputPath, max.p, allChipLLs)
    }
  }
  if (gocc) {
    if(updown) {
      cat("<A NAME='CC", stamp, "'></A>", sep="")
      cat("<H3>Cellular Component</H3>\n");
      hyperGTwilightHTML(LL_upreg, LL_downreg, "CC", outputPath, max.p, allChipLLs)
    } else {
      cat("<A NAME='CC", stamp, "'></A>", sep="")
      cat("<H3>Cellular Component</H3>\n");
      hyperGHTML(LL, "CC", outputPath, max.p, allChipLLs)
    }
  }
  if (kegg) {
    require(KEGG) || stop("Package KEGG unavailable")
    if(updown) {
      cat("<A NAME='KEGG", stamp, "'></A>", sep="")
      cat("<H3>Distribution of KEGG annotations</H3>\n");
      hyperKTwilightHTML(upreg, downreg, max.p)
    } else {
      cat("<A NAME='KEGG", stamp, "'></A>", sep="")
      cat("<H3>Distribution of KEGG annotations</H3>\n");
      hyperKHTML(x, max.p)
    }
  }

  cat('<script type="text/javascript">\n')
  cat('function openLink(aim, target) {\n')
  cat('window.open(aim, target, "width=600, height=600, dependent=no, menubar=no, status=no, toolbar=no, scrollbars=yes, resizable=yes");\n')
  cat('}\n</script>\n')


  
  sink()
  res <- ""
  for (l in readLines(.tmpFileName)) res <- paste(res, l, sep="")
  file.remove(.tmpFileName)
  data.pkgs <- c(chip, "GO")
  if (kegg) data.pkgs <- c(data.pkgs, "KEGG")
  res <- paste(res, logDataPackageVersion(data.pkgs), sep="")
  return(invisible(res))
}

writeColorLegend <- function(graphheader, max.p, upS, mxS, dnS,dfS,
                             c11,c12,c13,c14,c21,c22,c23,c24,
                             c31,c32,c33,c34,c41,c42,c43,c44) {
  # generate table with values
  low <- max.p/100
  mid <- max.p/10
  hig <- max.p
  tmpTab<-data.frame(
                 uplow=c(  0,low,mid,  0,  0,  0,low,low,low,mid,mid,mid,hig,hig,hig,hig),
                 uphig=c(low,mid,hig,low,low,low,mid,mid,mid,hig,hig,hig,  1,  1,  1,  1),
                 dnlow=c(hig,hig,hig,mid,low,  0,mid,low,  0,mid,low,  0,mid,low,  0,hig),
                 dnhig=c(  1,  1,  1,hig,mid,low,hig,mid,low,hig,mid,low,hig,mid,low,  1),
                 shape=c(upS,upS,upS,mxS,mxS,mxS,mxS,mxS,mxS,mxS,mxS,mxS,dnS,dnS,dnS,dfS),
                 color=c(c14,c24,c34,c13,c12,c11,c23,c22,c21,c33,c32,c31,c43,c42,c41,c44))

  # write dot-file for color legend
  cat(graphheader, file="color.legend.dot")
  for (i in nrow(tmpTab):1) {
    cat('n', i, ' [label="', as.character(tmpTab[i,1]), ' < p_up < ', 
                             as.character(tmpTab[i,2]), '\\n', 
                             as.character(tmpTab[i,3]), ' < p_down < ', 
                             as.character(tmpTab[i,4]), 
                  '" fillcolor="', as.character(tmpTab[i,6]), 
                  '" shape="', as.character(tmpTab[i,5]), '"];\n', 
        sep="", file="color.legend.dot", append=TRUE)
  }
  cat("}\n", file="color.legend.dot", append=TRUE)
  system("dot -Tpng < color.legend.dot > color.legend.png")
  return(invisible(tmpTab))
}






