makeCDNAlayout <- function(chipName) {
  # example for dataset Sorlie01

  # name of the SMD file with wanted layout
  xls_name = "2151.xls"

  # make layout
  opt <- list()

  # members of defs:
  # galfile   - the excel file the layout is based on
  # path      - the path of the excelfile, if you are
  #             in the same directory it is NULL
  # info.id   - a vector containing the names of the
  #             columns of the SMD file containing the
  #             informations for a gene (headers are case sensitive)
  # layout.id - a vector containing the names of the colums
  #             containing layout information should contain
  #             something like: Block = "sector",
  #             Row = "sectorcol" or "x grid coordinate (within sector)",
  #             Column = "sectorrow" or "y grid coordinate (within sector)",
  #             labels = "spot" ...
  defs <- list(galfile = xls_name, path = NULL,
    info.id = c("NAME", "Clone ID"), layout.id = c(Block = "SECTOR",
      Row = "SECTORCOL",
      Column = "SECTORROW"),
    labels = "SPOT", sep = "\t", quote = "\\", fill = TRUE,
    check.names = FALSE, as.is = TRUE, ncolumns = 4)
  
  gal.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.Galfile")))
  
  gal <- do.call("read.Galfile", gal.args)
  
  # gene names and clone ids
  gnames <- gal$gnames
  # marray layout class
  layout <- gal$layout
  
  # header_names stores the exact header names of the colums of the SMD file containing oligo, and measurement information you want to retrieve:
  # Gf         - green chanal intenity
  # Gb         - green chanal background
  # Rf         - red chanal intenity
  # Rb         - red chanal background
  # name_clone - name and clone id
  header_names <- list()
  header_names$Gf <- "CH1I_MEAN"
  header_names$Gb <- "CH1B_MEAN"
  header_names$Rf <- "CH2I_MEAN"
  header_names$Rb <- "CH2B_MEAN"
  header_names$name_clone <- c("NAME", "Clone ID")

  # file - the filename must correspond to the chip name (so sorlie_layout is not a very good choice... the chipname contains not the '.Rda' )
  # save the layout file into the "rawdata" folder of the dataset.
  save(gnames, layout, header_names, file=paste(chipName, ".rda", sep=""), compress=TRUE)
}
