optimTree <- function(hclust.obj,dist.obj,all=FALSE){

### converts a dendrogram into a matrix:
### 0 - leaf on left side,
### 1 - leaf on right side,
### 2 - leafs are not connected to this node
  tree2mat <- function(t){
    n <- t$merge
    m <- matrix(2,length(t$labels),nrow(n))
    
    up <- function(i,j,n){
      a <- n[i,j]
      b <- a[a>0]
      
      while (sum(a>0)>0){
        a <- a[a<0]
        a <- c(a,as.numeric(n[b,]))
        b <- a[a>0]
      }
      a
    }
    
    for (i in 1:nrow(n)){
      if (n[i,1]<0){m[-n[i,1],i] <- 0}
      if (n[i,1]>0){m[-up(i,1,n),i] <- 0}
      
      if (n[i,2]<0){m[-n[i,2],i] <- 1}
      if (n[i,2]>0){m[-up(i,2,n),i] <- 1}
    
    }
    
    rownames(m) <- 1:length(t$labels)
    m <- m[t$order,]
    m <- m[,rev(1:nrow(n))]
    m
  }
  
### converts matrix into dendrogram, needs original tree
  mat2tree <- function(m,orig){
    t <- orig
    t$order <- as.numeric(rownames(m))
    t
  }
  
### flips rows inside a matrix
  flip <- function(n,x){
    a <- which(x[,n]==0)
    b <- which(x[,n]==1)
    if (a[1]<b[1]){
      y <- x[c(b,a),]
    }
    if (b[1]<a[1]){
      y <- x[c(a,b),]
    }
    y
  }
  
### returns labeled combinations
  finish <- function(x,a){
    label <- NULL
    for (i in 1:nrow(x)){
      y <- matrix(x[i,],nrow=nrow(x),ncol=ncol(x),byrow=TRUE)
      z <- a==y
      z <- which(rowSums(z)==ncol(x))
      label <- c(label,names(z))
    }
    rownames(x) <- label
    x
  }
  
### computes cumulative distance between leafs going from left to right
  compDist <- function(t,d){
    d <- as.matrix(d)
    x <- t$labels[t$order]
    y <- 0
    for (i in 1:(length(x)-1)){
      y <- y+d[x[i],x[i+1]]
    }
    t$sumDist <- y
    t
  }

### sanity checks
  if (class(hclust.obj)!="hclust"){
    stop("First input has to be of class 'hclust'.")
  }
  if (class(dist.obj)!="dist"){
    stop("Second input has to be of class 'dist'.")
  }
  if (length(hclust.obj$order)!=attr(dist.obj,"Size")){
    stop("'hclust' and 'dist' input have to belong together.")
  }
  
  
  
  if (is.null(hclust.obj$labels)){
    hclust.obj$labels <- 1:length(hclust.obj$order)
  }

  a <- tree2mat(hclust.obj)

### let's list all possible fork switches
  r <- NULL
  for (i in 1:ncol(a)){r <- c(r,2^(ncol(a)-i))}
  
  mat <- rep(1,r[1])
  for (i in 2:length(r)){mat <- cbind(mat,rep(c(rep(1,r[i]),rep(0,r[i])),r[2]/r[i]))}
  
  x <- list()
  for (i in 1:nrow(mat)){x[[i]] <- a}
  
  for (j in 2:ncol(a)){
    for (i in 1:nrow(mat)){
      if (mat[i,j]==1){
        y <- which(x[[i]][,j]!=2)
        x[[i]][y,] <- flip(j,x[[i]][y,])
      }
    }
  }
  rm(i,j,y,r,mat)
  
  x <- lapply(x,finish,a)
  
  hc.new <- lapply(x,mat2tree,hclust.obj)
  hc.new <- lapply(hc.new,compDist,dist.obj)
  
### find the first ordering with minimal distance
  if (!all){
    s <- lapply(hc.new,function(x){x$sumDist})
    s <- unlist(s)
  
    hc.new <- hc.new[[which.min(s)]]
  }
  
  return(hc.new)
}
