### do the clusterplot
optimTree.plot <- function(x,optimize=TRUE,row.scale=TRUE,zlim=2,max.samp=15,barplot=FALSE,col="color1"){

  if (class(x)[1]=="exprSet"){
    xout <- exprs(x)
    rownames(xout) <- geneNames(x)
    colnames(xout) <- sampleNames(x)
    x <- xout
    rm(xout)
  }


  ### source code taken from clusterplot by Mark Wilkinson
  ### changed the very last line to optimTree.dendrogram(...)
  my.clusterplot <- function (hclust.x, hclust.y, X, labels.x = hclust.x$labels,
                              labels.y = hclust.y$labels, dend.x.opt = list(), dend.y.opt = list(),
                              labels.x.opt = list(), labels.y.opt = list(), image.opt = list(),
                              layout.opt = list(), cex = 1, spc.x = 1/2, spc.y = 1/2, indent = 3/4)
    {
      old.par <- par(no.readonly = TRUE)
      on.exit(par(old.par))
      require(stats)
      if (length(hclust.x$order) != ncol(X) || length(hclust.y$order) !=
          nrow(X)) {
        stop(paste("dim(X) must match number of leaves on the correpsonding",
                   "dendrogram.\nTranspose?"))
      }
      if (is.null(layout.opt$widths))
        layout.opt$widths <- c(0.2, 0.8)
      if (is.null(layout.opt$heights))
        layout.opt$heights <- c(0.2, 0.8)
      do.call("layout", c(list(mat = matrix(c(0, 2, 3, 1), 2),
                               respect = FALSE), layout.opt))
      frame()
      par(cex = cex, new = TRUE, bty = "n", xaxt = "n", yaxt = "n",
          ann = FALSE)
      labs.x <- as.matrix(as.matrix(labels.x)[hclust.x$order, ])
      labs.y <- as.matrix(as.matrix(labels.y)[hclust.y$order, ])
      nc.x <- ncol(labs.x)
      nr.x <- nrow(labs.x)
      nc.y <- ncol(labs.y)
      nr.y <- nrow(labs.y)
      inch2line <- function(x) {x * 2032/375 * 1/cex}
      widths.y <- inch2line(apply(labs.y, 2, function(xx) max(strwidth(xx, units = "inches", cex = par("cex")))))
      widths.x <- inch2line(apply(labs.x, 2, function(xx) max(strwidth(xx, units = "inches", cex = par("cex")))))
      widths.x <- rev(widths.x)
      if (nc.y > 1)
        widths.y[1:(nc.y - 1)] <- widths.y[1:(nc.y - 1)] + spc.y
      if (nc.x > 1)
        widths.x[1:(nc.x - 1)] <- widths.x[1:(nc.x - 1)] + spc.x
      xpos <- cumsum(widths.y) + indent
      ypos <- cumsum(widths.x) + indent
      mar4 <- max(xpos)
      mar1 <- max(ypos)
      xpos <- c(indent, xpos)[1:nc.y]
      ypos <- c(indent, ypos)[1:nc.x]
      tmp.mar <- par("mar")
      par(mar = c(tmp.mar[1] + mar1, 0, 0, tmp.mar[4] + mar4))
      tmp.X <- t(as.matrix(X[hclust.y$order, hclust.x$order]))
      if (!is.null(image.opt$zlim)) {
        tmp.X[tmp.X < min(image.opt$zlim)] <- min(image.opt$zlim)
        tmp.X[tmp.X > max(image.opt$zlim)] <- max(image.opt$zlim)
      }
      do.call("image.default", c(list(x = tmp.X), image.opt))
      box(which = "plot", bty = "o")
      at1 <- seq(0, 1, length = nr.y)
      axis(4, at = at1, labels = NA, tick = TRUE, line = 0, las = 2,
           cex = cex, yaxt = "s")
      at2 <- seq(0, 1, length = nr.x)
      axis(1, at = at2, labels = NA, tick = TRUE, line = 0, las = 2,
        cex = cex, xaxt = "s")
      if (is.null(labels.y.opt$col)) {
        cols.y <- matrix(par("col.axis"), nr.y, nc.y)
      }
      else {
        cols.y <- as.matrix(labels.y.opt$col)[hclust.y$order,
                                              ]
        labels.y.opt$col <- NULL
      }
      for (i in 1:nc.y) {
        do.call("mtext", c(list(text = labs.y[, i], side = 4, line = xpos[i], at = at1, las = 2, cex = cex, col = cols.y[,i]), labels.y.opt))
      }
      if (is.null(labels.x.opt$col)) {
        cols.x <- matrix(par("col.axis"), nr.x, nc.x)
      }
      else {
        cols.x <- as.matrix(labels.x.opt$col)[hclust.x$order,
                                              ]
        labels.x.opt$col <- NULL
      }
      for (i in 1:nc.x) {
        do.call("mtext", c(list(text = labs.x[, nc.x + 1 - i],
                                side = 1, line = ypos[i], at = at2, las = 2, cex = cex,
                                col = labels.x.opt$col[, i]), labels.x.opt))
      }
      par(yaxs = "i", xaxs = "i")
      par(mar = c(tmp.mar[1] + mar1, tmp.mar[2], 0, 0))
      do.call("plot", c(list(x = as.dendrogram(hclust.y), horiz = TRUE),dend.y.opt))
      par(mar = c(0, 0, tmp.mar[3], tmp.mar[4] + mar4))
      do.call("plot", c(list(x = optimTree.dendrogram(hclust.x), horiz = FALSE),dend.x.opt))
    }



  if ( (!is.numeric(x)) | (is.null(dim(x))) ){
    stop("First input must be numerical matrix with genes in rows and samples in columns.")
  }

  if (is.null(colnames(x))){
    colnames(x) <- 1:ncol(x)
  }
  if (is.null(rownames(x))){
    rownames(x) <- 1:nrow(x)
  }
  
  geneDist <- dist(x)
  sampDist <- dist(t(x))

  geneHclust <- hclust(geneDist,method="average")
  sampHclust <- hclust(sampDist,method="average")

  if (optimize){
    if (ncol(x)<=max.samp){
      sampHclust <- optimTree(sampHclust,sampDist)
    }
    else {print("No optimization possible due to large sample size. Maybe enlarge 'max.samp'.")}
  }


  ### some color schemes
  if (col=="color3"){
    color <- myPalette(low="green", high="red", mid="black", k=512)
  }
  if (col=="color2"){
    color <- myPalette(low="blue", high="yellow", mid="black", k=512)
  }
  if (col=="color1"){
    lsc <- c(0,(1:255/255)^(1/3)*100)
    color <- c(rgb(r=rev(lsc),g=rev(lsc),b=0,max=100),rgb(r=0,g=0,b=lsc,max=100))
    color <- rev(color)
  }
  
  layout.opt <- list()
  layout.opt$widths <- c(0.2, 0.8)
  layout.opt$heights <- c(0.1, 0.9)

  dend.x.opt <- list()
  dend.x.opt$yaxt <- "n"

  labels.x.opt <- list()
  labels.x.opt$las <- 1

  if (row.scale){x <- t(scale(t(x)))}

  b <- c(min(-max(abs(range(x))),-zlim-0.0001),seq(-zlim,zlim,length=511),max(max(abs(range(x))),zlim+0.0001))
  
  if (!barplot){
    par(mar=c(1, 1, 1, 2)+.1)
    my.clusterplot(sampHclust,
                   geneHclust,
                   x,
                   image.opt=list(col=color,breaks=b),
                   cex=min(1,75/max(dim(x))),
                   layout.opt=layout.opt,
                   dend.x.opt=dend.x.opt,
                   dend.y.opt=dend.x.opt,
                   labels.x.opt=labels.x.opt,
                   )
  }

  if (barplot){
    par(mar=c(2, 1, 1, 1)+.5)
    ColorBar(b,col=color)
  }
}
