\name{MvsA.cpd}
\alias{MvsA.cpd}
\title{Rank-vs-Residual Plots}
\description{
  Uses hexbin to draw rank-vs-residuals plots with loess and running
  median average deviation.
}
\usage{
MvsA.cpd(a.x, a.y, call = "", filename = NULL, sample.size = 10000, 
         win.size = 1000, win.step = trunc(length(a.x)/50), ...)
}
\arguments{
  \item{a.x}{values of considered chip.}
  \item{a.y}{values of reference chip, e.g. median over all chips.}
  \item{call}{whether the data is from normalization of summary.}
  \item{filename}{writes a bitmap/pdf if this argument is
      specified. The filename is completed with a file extension according
      to the type required and a postfix according to \code{call}.}
  \item{sample.size}{How many points are sampled for running MAD}
  \item{win.size}{how large is the average window for running MAD}
  \item{win.step}{step size for running MAD}
  \item{\dots}{further argument passed to 'bitmap'}
}
\details{
  Uses hexbin to draw a density plot rather than a scatter plot. We
  expect equally distributed residuals for good quality
  chips. Therefore, the blue dots representing median average deviation
  from residuals of a running window should be located on a horizontal
  line. The read line represents a loess fit and whould be close to zero
  for the whole x-range.
}
\author{Stefan Bentink}
\note{
  Use the additional argument type="pdfwrite" to generate PDF.
}
\seealso{\code{\link{bitmap}}}
\examples{
library(vsn)
library(golubEsets)
data(golubTest)

expr.mat <- exprs(vsn(golubTest, describe.preprocessing=FALSE))
expr.med <- apply(expr.mat, 1, median)
MvsA.cpd(expr.mat[,1], expr.med)
}
\keyword{hplot}
