\name{boxplt.cpd}
\alias{boxplt.cpd}
\alias{boxplt.plmset.cpd}
\title{Boxplot of Expression Levels}
\description{
  Draws a boxplot for each chip given in a gene expression matrix or
  standard errors given in plmset-object
}
\usage{
boxplt.cpd(fr, filename = NULL, pointsize = 14, height = 1.6 + ncol(fr) * pointsize/48, 
           call = "", ...)
boxplt.plmset.cpd(plmset, filename = NULL, pointsize = 14, 
                  height = 1.6 + ncol(coefs(plmset)) * pointsize/48, ...)
}
\arguments{
  \item{fr}{a gene expression matrix.}
  \item{plmset}{affyPLM-object of which standard error estimates are extracted.}
  \item{filename}{store a bitmap/pdf here is this argument is specified.}
  \item{pointsize}{gives the font size}
  \item{height}{height in inches}
  \item{call}{either 'summary' or 'normalized'}
  \item{\dots}{further arguments passed to bitmap command for writing files.}
}
\details{
  Draws a boxplot of gene expression levesl after summary or after
  normalization. Given the hypothesis that gene expression levels are
  distributed simmilarly across chips from the same experiment, we want
  to see similar boxplots as well.

  \code{boxplt.plmset.cpd} does the same for estimated standard errors
  on the residuals. We want to see low residuals, because this means
  that the fitting of the linear model went fine.
}
\author{Stefan Bentink}
\note{
  Use type="pdfwrite" as parameter to generate PDF
}
\seealso{\code{\link{bitmap}}}
\examples{
library(golubEsets)
data(golubTest)
expr.mat <- exprs(vsn(golubTest, describe.preprocessing=FALSE))
boxplt.cpd(expr.mat)
}
\keyword{hplot}
