\name{gokeggLister}
\alias{gokeggLister}
\title{Evaluate Overrepresentation of GO Terms and Pathways in Gene Lists}
\description{
  Uses hypergeometric tests to evaluate overrepresentation of KEGG pathway and GO
  annotations in gene lists.
}
\usage{
gokeggLister(x, chip, groupname="first group", ngenes = NULL, max.q = NULL,
             gomf = TRUE, gobp = TRUE, gocc = TRUE, kegg = TRUE, 
             outputPath = ".", max.p = 0.01, links = FALSE, legend = FALSE, 
             allChipLLs=NULL, is.upreg=NULL, bw.colors=FALSE, fontpath=NULL)
}
\arguments{
  \item{x}{a vector of probe-set identifiers as character strings or a twilight object if 
    \code{allChipLLs} is NULL. Otherwise \code{x} is expected to contain numeric LocusLink 
    identifiers}
  \item{chip}{a character string giving the name of a chip, i.e. its annotation library. 
    This is an Affymetrix chip name if \code{allChipLLs} is null and one of 
    \code{humanLLMappings}, \code{mouseLLMappings} or \code{ratLLMappings} otherwise.}
  \item{groupname}{Specifies the name of the group, where the genes used for the analysis 
    came from.}
  \item{ngenes}{number of genes taken into account}
  \item{max.q}{take all genes with q-values lower than max.q into account. Only valid
    when x is a twilight object.}
  \item{gomf}{logical, TRUE if the molecular function branch of GO is to be evaluated.}
  \item{gobp}{logical, TRUE if the biological process branch of GO is to be evaluated.}
  \item{gocc}{logical, TRUE if the cellular component branch of GO is to be evaluated.}
  \item{kegg}{logical, TRUE if the KEGG pathways are to be evaluated.}
  \item{outputPath}{where to store the bitmaps for graphs}
  \item{max.p}{numerical, defines the maximal p-value tolerated for GO/KEGG nodes 
    considered as overrepresented}
  \item{links}{Flags if external link sites for every GO graph node should be created.}
  \item{legend}{Flags if external legend site for the GO graphs should be created.}
  \item{allChipLLs}{is NULL if \code{x} is a list of probe-sets from an Affymetrix chip. 
    For other microarrays, this is a complete list of numeric LocusLink identifiers for 
    all reporters.}
  \item{is.upreg}{for each element of \code{x} contains a logical value specifying whether
    the corresponding probe-set/reporter is up regulated in \code{groupname}.}
  \item{bw.colors}{If FALSE up- and down-regulation overrepresentation is represented in 
    graphs by shades of red and green (default). If set to TRUE, this is shown using 
    different shapes of nodes.}
  \item{fontpath}{is used by 'dot' to get fonts for the drawing of graphs. The default 
    (NULL) tries to guess in a few standard paths.}
}
\details{
  Hypergeometric tests are performed for each GO term of selected branches and for all
  KEGG pathways if required. These tests compute p-values to evaluate the likelyhood
  that the corresponding number of annotations is occurring in a random list of genes
  of the same size.

  If \code{upreg} is specified, the analysis is carried out separately for up and down
  regulated probe-sets/reporters. For twilight objects \code{gokeggLister} determines 
  automatically which probe-sets/reporters are up or down regulated.
}
\value{
  A string containing HTML code to be added on an HTML page.
}
\author{Claudio Lottaz <lottaz@molgen.mpg.de>}
\examples{
library(hgu95av2)
ids <- ls(hgu95av2GO)[1:20]
gokeggLister(ids, chip="hgu95av2", outputPath=NULL)

llids=unique(unlist(mget(ids,env=hgu95av2LOCUSID)))
allLLids=unique(unlist(as.list(hgu95av2LOCUSID)))
gokeggLister(llids, chip="humanLLMappings", outputPath=NULL, allChipLLs=allLLids)

library(humanLLMappings)
ids <- ls(humanLLMappingsLL2GO)[1:100]
all.ids <- ls(humanLLMappingsLL2GO)[1:1000]
gokeggLister(ids, chip="humanLLMappings", allChipLLs=all.ids)
}
\keyword{hplot}
