\name{optimTree}
\alias{optimTree}
\title{Optimal reordering of hclust objects}
\description{\code{optimTree} rearranges an \code{hclust} object such that the sum of pairwise distances between neighboring leafs is minimal.}
\usage{
optimTree(hclust.obj, dist.obj, all = FALSE)
}
\arguments{
  \item{hclust.obj}{An object of class \code{hclust}.}
  \item{dist.obj}{The distance object from which \code{hclust.obj} was generated.}
  \item{all}{Logical value. If \code{FALSE}, only the first dendrogram with minimal sum of distances is returned. If \code{TRUE}, a list containing all possible dendrogram permutations is returned.}
}
\details{The function completely enumerates all possible dendrogram rearrangements of the given \code{hclust} object. Note, that computation time increases with sample size. With n samples, the number of possible dendrograms is $2^(n-2)$.}
\value{Returns an \code{hclust} object with additional slot \code{sumDist} containing the corresponding sum of distances. If \code{all=TRUE}, returns a list of \code{hclust} objects.}
\references{}
\author{Stefanie Scheid <stefanie.scheid@molgen.mpg.de>}
\seealso{\code{\link{optimTree.plot}},\code{\link{optimTree.dendrogram}}}
\examples{
a <- rbind(rep(0,4),
           c(1,rep(0,3)),
           c(5,2,0,0),
           c(3,9,7,0))

a <- as.dist(a)

hc.old <- hclust(a,method="average")
hc.new <- optimTree(hc.old,a)

plot(hc.old)
plot(hc.new)
print(hc.new$sumDist)
}
\keyword{graphs}
