\name{optimTree.plot}
\alias{optimTree.plot}
\title{Produces a clusterplot with optimal rearrangement of samples}
\description{Function is based on original \code{clusterplot} code by Mark Wilkinson. For a given expression matrix, hierarchical clustering with average linkage is performed on Euclidean distances. The samples (colums) are rearranged using \code{optimTree}.}
\usage{
optimTree.plot(x, optimize = TRUE, row.scale = TRUE, zlim = 2, max.samp = 15, barplot = FALSE, col = "color1")
}
\arguments{
  \item{x}{Matrix of expression values with genes in rows and samples in columns OR an \code{exprSet}.}
  \item{optimize}{Logical value. If \code{TRUE}, \code{optimTree} is used in succession with \code{hclust}.}
  \item{row.scale}{Logical value. If \code{TRUE}, genes are scaled before plotting to derive nicer colors.}
  \item{zlim}{Limit of plotted values. All values exceeding \code{+-zlim} are encoded by the brightest colors. Maybe increase the level if \code{row.scale=FALSE}.}
  \item{max.samp}{Maximal sample size. If there are more samples, no optimization is done. Note that computation time increases with sample size.}
  \item{barplot}{Logical value. If \code{TRUE}, the encoding color bar is plotted instead of a clusterplot.}
  \item{col}{Color schemes \code{color1} to \code{color3}.}
}
\value{Graphical output.}
\author{Stefanie Scheid <stefanie.scheid@molgen.mpg.de>, \code{clusterplot} code by Mark Wilkinson}
\seealso{\code{\link{optimTree}},\code{\link{optimTree.dendrogram}}}
\examples{
a <- matrix(rnorm(1000),ncol=10)
optimTree.plot(a)
optimTree.plot(a,barplot=TRUE)
}
\keyword{hplot}
