\name{weightedHammingDist}
\alias{weightedHammingDist}
\title{Distance to compare two splits}
\description{
  This function computes a distance between two bipartitions of the
  same data computed by the ISIS method.
}
\usage{
weightedHammingDist(ires, score.weight = 3)
}
\arguments{
  \item{ires}{Result returned by isis, a binary matrix with one row
    per split and one column per data point. The value at postion i, j in
    this matrix associates the sample i to cluster ires[i,j] in split i.
    In addition, the last column holds DLD scores for each split.}
  \item{score.weight}{The weights of splits are used to scale
    Hamming distances. Splits weights are exponentiated with this
    value for weighing a distance.}
}
\value{
  Returns a distance matrix containing distances between samples
according to their occurrence in splits. 
}
\author{Joern Toedling}
\note{integrated into compdiagTools by Claudio Lottaz}
\seealso{\code{\link{isis}}}
\examples{
# load data select only few genes
library(ALL)
data(ALL)
samples <- 63:128
vars <- apply(exprs(ALL), 1, var)
e <- exprs(ALL)[rank(vars)>12525, samples]

# determine isis splits and compute weights accordingly
library(isis)
splits <- isis(e)
w <- weightedHammingDist(splits)
}
\keyword{datagen}

